import http = require("http");
import https = require("https");
import { parse } from "url";
import HTMLParser from "node-html-parser";

const httpToData = (req: http.IncomingMessage): Promise<string> =>
  new Promise((resolve) => {
    let data = "";
    req.on("data", (chunk) => (data += chunk));
    req.on("end", () => resolve(data));
  });
const getCookie = (): Promise<string> =>
  new Promise((resolve) => {
    https.get("https://m.swimtimes.nl/nl/security/login", (res) => {
      httpToData(res).then((html) => {
        const cookies: { [key: string]: string } = {};
        res.headers["set-cookie"].forEach(function (rawCookie) {
          const cookieParts = rawCookie.split(";")[0].split("=");
          cookies[cookieParts.shift()] = cookieParts.join("=");
        });
        const cookieString = Object.keys(cookies)
          .map(function (cookie) {
            return cookie + "=" + cookies[cookie];
          })
          .join("; ");
        const root = HTMLParser.parse(html);
        const csrf_token = root
          .querySelector('[name="_csrf_token"]')
          .getAttribute("value");
        const message =
          "_username=aih29723%40ixaks.com&_password=aih29723%40ixaks.com&_csrf_token=" +
          csrf_token;
        console.log("sending", message);
        https
          .request(
            "https://m.swimtimes.nl/nl/security/login",
            {
              method: "POST",
              headers: {
                Cookie: cookieString,
                "Content-Type": "application/x-www-form-urlencoded",
                "Content-Length": message.length,
              },
            },
            function (res) {
              cookies.PHPSESSID = res.headers["set-cookie"][1]
                .split(";")[0]
                .split("=")[1];
              resolve(
                Object.keys(cookies)
                  .map(function (cookie) {
                    return cookie + "=" + cookies[cookie];
                  })
                  .join("; "),
              );
            },
          )
          .end(message);
      });
    });
  });
function parseTime(tijd: string): number {
  let [seconds, minuten, uren] = tijd.split(":").reverse().map(Number);
  if (!uren) uren = 0;
  if (!minuten) minuten = 0;
  if (!seconds) seconds = 0;
  return (uren * 60 + minuten) * 60 + seconds;
}
function pad(num: number, size: number) {
  let s = num + "";
  while (s.length < size) s = "0" + s;
  return s;
}
function toTime(totaal: number) {
  let minuten: number | string = Math.floor(totaal / 60);
  minuten = pad(minuten % 60, 2);
  let secondes: number | string = totaal % 60;
  const honderdsten = pad(Math.round((secondes % 1) * 100), 2);
  secondes = pad(Math.floor(secondes), 2);
  return minuten + ":" + secondes + "." + honderdsten;
}
async function last(swimmerId: number) {
  const cookie = await getCookie();
  const html: string = await new Promise((resolve) => {
    https.get(
      `https://m.swimtimes.nl/nl/athlete/${swimmerId}/widget`,
      { headers: { Cookie: cookie } },
      (res) => resolve(httpToData(res)),
    );
  });
  const root = HTMLParser.parse(html);
  const dates = root.querySelectorAll('[class="list-group my-1"]');
  const response = {};
  dates.forEach((date) => {
    const dateParts = date
      .querySelector('[class="list-group-item text-muted text-truncate"]')
      .innerText.split(" - ");
    dateParts.shift();
    const dateString = dateParts.join(" - ").trim();
    const afstanden = date.querySelectorAll(
      '[class="list-group-item list-group-item-action"]',
    );
    response[dateString] = afstanden.map((afstand) => {
      const slagParts = afstand
        .querySelector('[class="col text-truncate"]')
        .innerText.split(" ");
      const beginMeters = slagParts[0];
      let meters = +slagParts.shift();
      const slag = slagParts.join(" ");
      const tijd = parseTime(
        afstand.querySelector('[class="col-auto"]').innerText,
      );
      const tussentijden = [];
      let tussentijd = tijd;
      while (meters > 25 && (meters / 2) % 1 === 0) {
        meters = meters / 2;
        tussentijd = tussentijd / 2;
        tussentijden.push({ distance: meters, time: toTime(tussentijd) });
      }
      return {
        distance: beginMeters,
        slag,
        total: toTime(tijd),
        subtimes: tussentijden,
      };
    });
  });
  return response;
}
async function records(swimmerId: number) {
  const cookie = await getCookie();
  const message = "season%5Bseason%5D=0";
  const html: string = await new Promise((resolve) => {
    https
      .request(
        `https://m.swimtimes.nl/nl/athlete/${swimmerId}/record`,
        {
          method: "POST",
          headers: {
            Cookie: cookie,
            "Content-Type": "application/x-www-form-urlencoded",
            "Content-Length": message.length,
          },
        },
        (res) => resolve(httpToData(res)),
      )
      .end(message);
  });
  const root = HTMLParser.parse(html);
  const allRecords = root.querySelectorAll(
    '[class="list-group-item list-group-item-action"]',
  );
  return allRecords.map((record, i) => {
    let totaalAfstand = record.querySelector(
      '[class="col d-none d-md-block"]',
    ).innerText;
    if (totaalAfstand.length === 0)
      totaalAfstand = allRecords[i - 1].querySelector(
        '[class="col d-none d-md-block"]',
      ).innerText;
    let afstand = parseInt(totaalAfstand.split(" ")[0]);
    const meters = record.querySelector('[class="col-2"]').innerText;
    const tijd = parseTime(
      record.querySelector('[class="col-auto col-md"]').innerText,
    );
    const niveau = record.querySelector(
      '[class="col-1 text-center"]',
    ).innerText;
    const datum = record.querySelector('[class="col text-right"]').innerText;
    const stad = record.querySelectorAll('[class="col d-none d-md-block"]')[1]
      .innerText;
    const tussentijden = [];
    let tussentijd = tijd;
    const originalAfstand = afstand;
    while (afstand > 25 && (afstand / 2) % 1 === 0) {
      afstand = afstand / 2;
      tussentijd = tussentijd / 2;
      tussentijden.push({ distance: afstand, time: toTime(tussentijd) });
    }
    return {
      afstand: originalAfstand,
      meters,
      tijd: toTime(tijd),
      niveau,
      datum,
      stad,
      totaalAfstand,
      tussentijden,
    };
  });
}
const server = http.createServer(async function (req, response) {
  response.setHeader("Access-Control-Allow-Origin", "*");
  response.setHeader("Access-Control-Request-Method", "*");
  response.setHeader("Access-Control-Allow-Methods", "OPTIONS, GET");
  response.setHeader("Access-Control-Allow-Headers", "*");
  if (req.method === "OPTIONS") {
    response.writeHead(200);
    response.end();
    return;
  }
  const parts = parse(req.url, true).pathname.split("/");
  const reqType = parts[1];
  const swimmer = parts[2];
  if (!reqType) return "no reqType.";
  if (!swimmer) return "no swimmer.";
  const functions = {
    last,
    records,
  };
  if (!functions[reqType]) return response.end("invalid request.");
  const data = await functions[reqType](swimmer);
  response.end(JSON.stringify(data, null, 2));
});
server.listen(8080);
console.log("started!");
